//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------


/**
 * @file
 * Implementation for class TChessPiece
 */

#include "chesspiece.h"
#include "settings.h"

#include <pf/pflib.h>
#include <pf/model.h>

PFTYPEIMPL_DC(TChessPiece);

TChessPiece::TChessPiece() :
	mYaw(0),
	mLastUpdate(TPlatform::GetInstance()->Timer()),
	mDragging(false),
	mDragOrigin(0,0)
{
	mModel = TModel::Get("mesh/king.mesh");
	mTexture = TTexture::Get("mesh/white");

	memset(&mMat,0,sizeof(mMat));

	mMat.mcAmb.r = 0.2f;
	mMat.mcAmb.g = 0.2f;
	mMat.mcAmb.b = 0.2f;
	mMat.mcAmb.a = 1.0f;

	mMat.mcDiff.r = 1.0f;
	mMat.mcDiff.g = 1.0f;
	mMat.mcDiff.b = 1.0f;
	mMat.mcDiff.a = 1.0f;

	mMat.mcSpec.a = 1.0f;
	mMat.mcSpec.r = 0.75f;
	mMat.mcSpec.g = 0.75f;
	mMat.mcSpec.b = 0.75f;
	mMat.mcEmit.a = 0.75f;

	mMat.mfPower = 10.0f;

	memset(&mLight,0,sizeof(mLight));
	mLight.mDiff.r = 1.0f;
	mLight.mDiff.g = 0.0f;
	mLight.mDiff.b = 0.0f;
	mLight.mDiff.a = 1.0f;
	mLight.mAmb.r = 0.1f;
	mLight.mAmb.g = 0.1f;
	mLight.mAmb.b = 0.1f;
	mLight.mAmb.a = 1.0f;
	mLight.mSpec.a = 1.0f;
	mLight.mSpec.r = 0.1f;
	mLight.mSpec.g = 0.1f;
	mLight.mSpec.b = 0.1f;

	mLight.mDir.z = 1 ;

	memset(&mSpinLight,0,sizeof(mSpinLight));
	mSpinLight.mDiff.r = 0.0F;
	mSpinLight.mDiff.g = 0.0F;
	mSpinLight.mDiff.b = 0.0F;
	mSpinLight.mDiff.a = 1.0F;
	mSpinLight.mAmb.r = 0.0f;
	mSpinLight.mAmb.g = 0.0f;
	mSpinLight.mAmb.b = 0.0f;
	mSpinLight.mAmb.a = 1.0f;
	mSpinLight.mSpec.r = 0.5f;
	mSpinLight.mSpec.g = 1.0f;
	mSpinLight.mSpec.b = 0.0f;
	mSpinLight.mSpec.a = 1.0f;
}

TChessPiece::~TChessPiece()
{
	// Log a high score
	if (TSettings::GetInstance() && TSettings::GetInstance()->GetHiscores())
		TSettings::GetInstance()->GetHiscores()->LogScore(rand() % 10000, true, NULL);
}

/**
 * Draw the window. Derived classes will override this
 * function and provide the draw functionality.
 */
void TChessPiece::Draw()
{
	// Getting a copy of TPlatform for convenience
	TRenderer * r = TRenderer::GetInstance();
	TRect screenRect ;
	GetWindowRect(&screenRect);

	{
		TBegin3d begin3d;

		// Set up our perspective projection matrix
		r->SetPerspectiveProjection(0.1F,100.0F,PI/5.0f);

		// Set up our rendering texture
		r->SetTexture(mTexture);

		// Set up our light
		r->SetLight(0,&mLight);

		// Set up our material
		r->SetMaterial(&mMat);

		TMat4 localMatrix ;
		localMatrix.Identity();

		// Build transform for yaw about board-y axis
		localMatrix.RotateY(mYaw);

		// Build transform for pitch about view-x axis

		TMat4 pitch, view;
		TRenderer::GetInstance()->GetViewMatrix(&view);
		pitch.RotateAxis((TVec3)view[0], PI/6);

		localMatrix = pitch*localMatrix ;

		localMatrix[3][0] = 0;
		localMatrix[3][1] = -0.75;
		localMatrix[3][2] = 3;

		mSpinLight.mDir.x = cos(mYaw*2) ;
		mSpinLight.mDir.y = sin(mYaw*3) ;
		mSpinLight.mDir.z = cos(mYaw*2) ;
		mSpinLight.mDir.Normalize();

		// Set up our light
		r->SetLight(1,&mSpinLight);
		r->SetWorldMatrix(&localMatrix);

		if (mModel)
		{
			mModel->Draw();
		}
		begin3d.Done();
	}
	{
		TBegin2d begin2d;
		TColor color(1,0,0,1);

		GetClientRect(&screenRect);
		r->FillRect(TURect(screenRect.mX1,screenRect.mY1,screenRect.mX1+2,screenRect.mY2),color);
		r->FillRect(TURect(screenRect.mX1,screenRect.mY1,screenRect.mX2,screenRect.mY1+2),color);
		r->FillRect(TURect(screenRect.mX2-2,screenRect.mY1,screenRect.mX2,screenRect.mY2),color);
		r->FillRect(TURect(screenRect.mX1,screenRect.mY2-2,screenRect.mX2,screenRect.mY2),color);
		begin2d.Done();
	}
}

/**
 * Initialize the Window. Called by the system only in
 * Lua initialization.
 *
 * When you create your own custom window, this is where you put
 * your own custom initialization that needs to happen before
 * children are created. Fundamental window initialization is
 * handled in every class by this function, so <b>when you
 * override this function you almost always want to call your
 * base class to handle base class initialization.</b>
 */
void TChessPiece::Init(TWindowStyle & /*style*/)
{
	StartWindowAnimation(16);
}

/**
 * Called if you have initiated a window animation with
 * TWindow::StartWindowAnimation.
 *
 * @return True to continue animating. False to stop.
 */
bool TChessPiece::OnTaskAnimate()
{
	float elapsedTime = (float)(TPlatform::GetInstance()->Timer() - mLastUpdate)/1000.0f;
	mLastUpdate = TPlatform::GetInstance()->Timer();

	mYaw += 0.7f*elapsedTime;
	if (mYaw > (2*PI)) mYaw -=(2*PI) ;

	TWindowManager::GetInstance()->InvalidateScreen();
	return true;
}

/**
 * Mouse down handler.
 *
 * @param point      Location of mouse press in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TChessPiece::OnMouseDown(const TPoint &point)
{
	mDragOrigin = point;
	mDragging = true;
	TWindowManager::GetInstance()->AddMouseListener(this);
	return true;
}

/**
 * Notification that the mouse has left the window.
 *
 * @warning This message is only sent if SetCapture()
 * has been called for this window previously.
 *
 * @return True if handled.
 */
bool TChessPiece::OnMouseLeave()
{
	mDragging = false;
	TWindowManager::GetInstance()->RemoveMouseListener(this);
	return true;
}

/**
 * Mouse motion handler.
 *
 * @param point      Location of mouse in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TChessPiece::OnMouseMove(const TPoint &point)
{
	if (mDragging)
	{
		SetWindowPos( GetParentRelativeRect().GetTopLeft()+point-mDragOrigin );
		return true;
	}
	return false;
}

/**
 * Mouse up handler.
 *
 * @param point      Location of mouse release in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TChessPiece::OnMouseUp(const TPoint & /*point*/)
{
	OnMouseLeave();
	return true;
}

/**
 * Extended mouse button handler.
 *
 * @param point  Location of mouse event in client coordinates.
 * @param event Event that happened.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TChessPiece::OnExtendedMouseEvent(const TPoint &point, TPlatform::ExtendedMouseEvents event)
{
	switch (event)
	{
		case TPlatform::kMouseScrollUp:
			SetWindowPos( GetParentRelativeRect().GetTopLeft()+TPoint(0,-5) );
			break;
		case TPlatform::kMouseScrollDown:
			SetWindowPos( GetParentRelativeRect().GetTopLeft()+TPoint(0,5) );
			break;
		case TPlatform::kMouseScrollLeft:
			SetWindowPos( GetParentRelativeRect().GetTopLeft()+TPoint(-5,0) );
			break;
		case TPlatform::kMouseScrollRight:
			SetWindowPos( GetParentRelativeRect().GetTopLeft()+TPoint(5,0) );
			break;
	}
	return true;
}
